 function _nullishCoalesce(lhs, rhsFn) { if (lhs != null) { return lhs; } else { return rhsFn(); } } function _optionalChain(ops) { let lastAccessLHS = undefined; let value = ops[0]; let i = 1; while (i < ops.length) { const op = ops[i]; const fn = ops[i + 1]; i += 2; if ((op === 'optionalAccess' || op === 'optionalCall') && value == null) { return undefined; } if (op === 'access' || op === 'optionalAccess') { lastAccessLHS = value; value = fn(value); } else if (op === 'call' || op === 'optionalCall') { value = fn((...args) => value.call(lastAccessLHS, ...args)); lastAccessLHS = undefined; } } return value; }// 

import {





  SwapCurrencyError
} from 'edge-core-js/types'

/**
 * Ensures that a date is in the future by at least the given amount.
 */
export function ensureInFuture(
  date,
  marginSeconds = 30
) {
  if (date == null) return
  const target = Date.now() + marginSeconds * 1000
  return target < date.valueOf() ? date : new Date(target)
}

export function makeSwapPluginQuote(
  request,
  fromNativeAmount,
  toNativeAmount,
  tx,
  destinationAddress,
  pluginId,
  isEstimate = false,
  expirationDate,
  quoteId
) {
  const { fromWallet } = request

  const out = {
    fromNativeAmount,
    toNativeAmount,
    networkFee: {
      currencyCode: fromWallet.currencyInfo.currencyCode,
      nativeAmount:
        tx.parentNetworkFee != null ? tx.parentNetworkFee : tx.networkFee
    },
    destinationAddress,
    pluginId,
    expirationDate,
    quoteId,
    isEstimate,
    async approve() {
      const signedTransaction = await fromWallet.signTx(tx)
      const broadcastedTransaction = await fromWallet.broadcastTx(
        signedTransaction
      )
      await fromWallet.saveTx(signedTransaction)

      return {
        transaction: broadcastedTransaction,
        orderId: quoteId,
        destinationAddress
      }
    },

    async close() {}
  }
  return out
}








export const getCodes = (request) => ({
  fromMainnetCode: request.fromWallet.currencyInfo.currencyCode,
  toMainnetCode: request.toWallet.currencyInfo.currencyCode,
  fromCurrencyCode: request.fromCurrencyCode,
  toCurrencyCode: request.toCurrencyCode
})

const getPluginIds = (
  request
) => ({
  fromPluginId: request.fromWallet.currencyInfo.pluginId,
  toPluginId: request.toWallet.currencyInfo.pluginId
})






/**
 * Throws if either currency code has been disabled by the plugin
 */
export function checkInvalidCodes(
  invalidCodes,
  request,
  swapInfo
) {
  const { fromPluginId, toPluginId } = getPluginIds(request)
  const {
    fromMainnetCode,
    toMainnetCode,
    fromCurrencyCode,
    toCurrencyCode
  } = getCodes(request)

  function check(
    direction,
    pluginId,
    main,
    token
  ) {
    switch (invalidCodes[direction][pluginId]) {
      case undefined:
        return false
      case 'allCodes':
        return true
      case 'allTokens':
        return main !== token
      default:
        return invalidCodes[direction][pluginId].some(code => code === token)
    }
  }

  if (
    check('from', fromPluginId, fromMainnetCode, fromCurrencyCode) ||
    check('to', toPluginId, toMainnetCode, toCurrencyCode)
  )
    throw new SwapCurrencyError(
      swapInfo,
      request.fromCurrencyCode,
      request.toCurrencyCode
    )
}







/**
 * Transcribes requested currency codes into plugin compatible unique IDs
 */
export function safeCurrencyCodes(
  transcriptionMap,
  request,
  toLowerCase = false
)


 {
  const { fromPluginId, toPluginId } = getPluginIds(request)
  const { fromCurrencyCode, toCurrencyCode } = getCodes(request)

  const out = {
    safeFromCurrencyCode: fromCurrencyCode,
    safeToCurrencyCode: toCurrencyCode
  }
  if (_optionalChain([transcriptionMap, 'access', _ => _[fromPluginId], 'optionalAccess', _2 => _2[request.fromCurrencyCode]])) {
    out.safeFromCurrencyCode =
      transcriptionMap[fromPluginId][request.fromCurrencyCode]
  }
  if (_optionalChain([transcriptionMap, 'access', _3 => _3[toPluginId], 'optionalAccess', _4 => _4[request.toCurrencyCode]])) {
    out.safeToCurrencyCode =
      transcriptionMap[toPluginId][request.toCurrencyCode]
  }

  if (toLowerCase)
    Object.keys(out).forEach(key => {
      out[key] = out[key].toLowerCase()
    })

  return out
}





/**
 * Returns all four codes with mainnet transcription
 */
export const getCodesWithMainnetTranscription = (
  request,
  transcriptionMap
) => {
  const {
    fromCurrencyCode,
    toCurrencyCode,
    fromMainnetCode,
    toMainnetCode
  } = getCodes(request)
  return {
    fromMainnetCode:
      _nullishCoalesce(transcriptionMap[request.fromWallet.currencyInfo.pluginId], () => (
      fromMainnetCode)),
    toMainnetCode:
      _nullishCoalesce(transcriptionMap[request.toWallet.currencyInfo.pluginId], () => ( toMainnetCode)),
    fromCurrencyCode: fromCurrencyCode,
    toCurrencyCode: toCurrencyCode
  }
}
